<?php

namespace Drupal\disk_space_alert\Controller;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\ByteSizeMarkup;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\disk_space_alert\Service\DiskSpaceAlertService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller for Disk Space Alert.
 */
class DiskSpaceAlertController extends ControllerBase {

  /**
   * The disk space alert service.
   *
   * @var \Drupal\disk_space_alert\Service\DiskSpaceAlertService
   */
  protected $diskSpaceAlertService;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The string translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $translation;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a DiskSpaceAlertController object.
   *
   * @param \Drupal\disk_space_alert\Service\DiskSpaceAlertService $diskSpaceAlertService
   *   The disk space alert service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   */
  public function __construct(DiskSpaceAlertService $diskSpaceAlertService, MessengerInterface $messenger, AccountInterface $currentUser, TranslationInterface $translation, TimeInterface $time, ConfigFactoryInterface $configFactory) {
    $this->diskSpaceAlertService = $diskSpaceAlertService;
    $this->messenger = $messenger;
    $this->currentUser = $currentUser;
    $this->translation = $translation;
    $this->time = $time;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): DiskSpaceAlertController {
    return new static(
      $container->get('disk_space_alert.service'),
      $container->get('messenger'),
      $container->get('current_user'),
      $container->get('string_translation'),
      $container->get('datetime.time'),
      $container->get('config.factory')
    );
  }

  /**
   * Returns the disk space information in JSON format.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The disk space information in JSON format.
   */
  public function getDiskSpace(): JsonResponse {
    if (!$this->currentUser->hasPermission('administer site configuration')) {
      return new JsonResponse(['error' => 'Access denied'], Response::HTTP_FORBIDDEN);
    }

    $config = $this->configFactory->get('disk_space_alert.settings');
    if (!$config->get('enabled')) {
      return new JsonResponse(['error' => 'Disk space alert is disabled.'], Response::HTTP_FORBIDDEN);
    }

    $disk_info = $this->checkDiskSpace();
    return new JsonResponse($this->convertToCamelCase($disk_info));
  }

  /**
   * Manually triggers a disk space check.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response.
   */
  public function manualCheck(): RedirectResponse {
    $config = $this->configFactory->get('disk_space_alert.settings');
    if (!$config->get('enabled')) {
      $this->messenger->addError($this->translation->translate('Disk space alert is disabled.'));
      return new RedirectResponse('/admin/config/system/disk-space-alert');
    }

    $this->diskSpaceAlertService->checkDiskSpace();
    $this->messenger->addStatus($this->translation->translate('Disk space check has been performed.'));

    return new RedirectResponse('/admin/config/system/disk-space-alert');
  }

  /**
   * Checks the disk space.
   *
   * @return array
   *   An array of disk space information.
   */
  private function checkDiskSpace(): array {
    $config = $this->configFactory->get('disk_space_alert.settings');
    $total_space = disk_total_space("/");
    $free_space = disk_free_space("/");
    $used_space = $total_space - $free_space;
    $used_percentage = round(($used_space / $total_space) * 100, 2);
    $available_percentage = round(($free_space / $total_space) * 100, 2);

    $threshold = (float) $config->get('threshold');
    $is_above_threshold = $used_percentage > $threshold;
    $above_threshold_percentage = round($used_percentage - $threshold, 2);
    $above_threshold_size = $is_above_threshold ? ($total_space * $above_threshold_percentage / 100) : 0;

    return [
      'diskName' => '/',
      'totalSpace' => ByteSizeMarkup::create($total_space, $this->currentUser->getPreferredLangcode())->render(),
      'totalBytes' => $total_space,
      'availableSpace' => ByteSizeMarkup::create($free_space, $this->currentUser->getPreferredLangcode())->render(),
      'availableBytes' => $free_space,
      'availablePercentage' => $available_percentage,
      'usedSpace' => ByteSizeMarkup::create($used_space, $this->currentUser->getPreferredLangcode())->render(),
      'usedBytes' => $used_space,
      'usedPercentage' => $used_percentage,
      'threshold' => $threshold,
      'isAboveThreshold' => $is_above_threshold,
      'aboveThresholdSize' => ByteSizeMarkup::create($above_threshold_size, $this->currentUser->getPreferredLangcode())->render(),
      'aboveThresholdBytes' => $above_threshold_size,
      'aboveThresholdPercentage' => $above_threshold_percentage,
      'timestamp' => $this->time->getRequestTime(),
    ];
  }

  /**
   * Converts array keys to camel case.
   *
   * @param array $data
   *   The data array.
   *
   * @return array
   *   The array with camel case keys.
   */
  private function convertToCamelCase(array $data): array {
    $result = [];
    foreach ($data as $key => $value) {
      $key = lcfirst(str_replace(' ', '', ucwords(str_replace('_', ' ', $key))));
      $result[$key] = $value;
    }
    return $result;
  }

}
