<?php

namespace Drupal\disk_space_alert\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines a form that configures disk space alert settings.
 */
class DiskSpaceAlertSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['disk_space_alert.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'disk_space_alert_settings_form';
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('disk_space_alert.settings');

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $config->get('enabled'),
    ];

    $form['threshold'] = [
      '#type' => 'number',
      '#title' => $this->t('Disk space threshold (%)'),
      '#default_value' => $config->get('threshold'),
      '#min' => 0,
      '#max' => 100,
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['notify_admin'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Notify site admin by email'),
      '#default_value' => $config->get('notify_admin'),
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['post_url'] = [
      '#type' => 'url',
      '#title' => $this->t('POST request URL'),
      '#default_value' => $config->get('post_url'),
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['post_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom POST message'),
      '#default_value' => $config->get('post_message'),
      '#description' => $this->t('Use tokens like @diskName, @totalSpace, @totalBytes, @availableSpace, @availableBytes, @availablePercentage, @occupiedSpace, @occupiedBytes, @occupiedPercentage, @threshold, @isAboveThreshold, @aboveThresholdSize, @aboveThresholdBytes, @aboveThresholdPercentage, @timestamp. The message can be in JSON, XML, or plain text format.'),
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
          ':input[name="post_url"]' => ['filled' => TRUE],
        ],
      ],
    ];

    $form['avoid_duplicate_triggers'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Avoid duplicate triggers'),
      '#default_value' => $config->get('avoid_duplicate_triggers'),
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Form submission handler.
   *
   * @param array &$form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('disk_space_alert.settings');
    $old_threshold = $config->get('threshold');
    $new_threshold = $form_state->getValue('threshold');

    $config->set('enabled', $form_state->getValue('enabled'))
      ->set('threshold', $new_threshold)
      ->set('notify_admin', $form_state->getValue('notify_admin'))
      ->set('post_url', $form_state->getValue('post_url'))
      ->set('post_message', $form_state->getValue('post_message'))
      ->set('avoid_duplicate_triggers', $form_state->getValue('avoid_duplicate_triggers'));

    if ($old_threshold != $new_threshold) {
      $config->set('last_exceeded_value', 0);
    }

    $config->save();

    parent::submitForm($form, $form_state);
  }

}
