<?php

namespace Drupal\disk_space_alert\Service;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\ByteSizeMarkup;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;

/**
 * Service for Disk Space Alert.
 */
class DiskSpaceAlertService {
  use StringTranslationTrait;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected MailManagerInterface $mailManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected MessengerInterface $messenger;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $httpClient;

  /**
   * The logger channel.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected TimeInterface $time;

  /**
   * Constructs a DiskSpaceAlertService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory.
   * @param \Drupal\Core\Mail\MailManagerInterface $mailManager
   *   The mail manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \GuzzleHttp\ClientInterface $httpClient
   *   The HTTP client.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger channel.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(ConfigFactoryInterface $configFactory, MailManagerInterface $mailManager, MessengerInterface $messenger, ClientInterface $httpClient, LoggerInterface $logger, LanguageManagerInterface $languageManager, TimeInterface $time) {
    $this->configFactory = $configFactory;
    $this->mailManager = $mailManager;
    $this->messenger = $messenger;
    $this->httpClient = $httpClient;
    $this->logger = $logger;
    $this->languageManager = $languageManager;
    $this->time = $time;
  }

  /**
   * Checks the disk space.
   */
  public function checkDiskSpace(): void {
    $config = $this->configFactory->get('disk_space_alert.settings');

    if (!$config->get('enabled')) {
      $this->logger->info('Disk space alert is disabled.');
      return;
    }

    $this->logger->info('Running disk space check');

    $disk_info = $this->getDiskSpace();
    $disk_info['threshold'] = $config->get('threshold');
    $disk_info['is_above_threshold'] = $disk_info['used_percentage'] > $disk_info['threshold'];
    $disk_info['above_threshold_percentage'] = round($disk_info['used_percentage'] - $disk_info['threshold'], 2);
    $disk_info['above_threshold_size'] = $disk_info['is_above_threshold'] ? ($disk_info['total_space_bytes'] * $disk_info['above_threshold_percentage'] / 100) : 0;
    $disk_info['timestamp'] = $this->time->getRequestTime();

    $last_exceeded_value = $config->get('last_exceeded_value');
    $avoid_duplicate_triggers = $config->get('avoid_duplicate_triggers');

    if ($disk_info['is_above_threshold']) {
      if ($disk_info['above_threshold_percentage'] < $last_exceeded_value || !$avoid_duplicate_triggers) {
        // State changed to above threshold, send notifications.
        $this->logger->info('Disk space above threshold: @above_threshold_percentage%', ['@above_threshold_percentage' => $disk_info['above_threshold_percentage']]);
        if ($config->get('notify_admin')) {
          $this->sendEmailNotification($disk_info);
        }
        if ($config->get('post_url')) {
          $this->sendPostRequest($disk_info, $config->get('post_url'), $config->get('post_message'));
        }

        // Update the exceeded value to reflect the current state.
        $this->configFactory->getEditable('disk_space_alert.settings')->set('last_exceeded_value', $disk_info['above_threshold_percentage'])->save();
      }
    }
    else {
      // Reset the exceeded value when disk space is below the threshold.
      $this->configFactory->getEditable('disk_space_alert.settings')->set('last_exceeded_value', 0)->save();
    }
  }

  /**
   * Retrieves disk space information.
   *
   * @return array
   *   The disk space information.
   */
  private function getDiskSpace(): array {
    // This function would contain the actual logic to check disk space.
    // For simplicity, we'll use placeholder values here.
    $total_space = disk_total_space("/");
    $free_space = disk_free_space("/");
    $used_space = $total_space - $free_space;
    $used_percentage = round(($used_space / $total_space) * 100, 2);
    $available_percentage = round(($free_space / $total_space) * 100, 2);

    return [
      'disk_name' => '/',
      'total_space' => ByteSizeMarkup::create($total_space, $this->languageManager->getDefaultLanguage()->getId())->render(),
      'total_space_bytes' => $total_space,
      'free_space' => ByteSizeMarkup::create($free_space, $this->languageManager->getDefaultLanguage()->getId())->render(),
      'free_space_bytes' => $free_space,
      'used_space' => ByteSizeMarkup::create($used_space, $this->languageManager->getDefaultLanguage()->getId())->render(),
      'used_space_bytes' => $used_space,
      'used_percentage' => $used_percentage,
      'available_percentage' => $available_percentage,
    ];
  }

  /**
   * Sends an email notification.
   *
   * @param array $disk_info
   *   The disk space information.
   */
  private function sendEmailNotification(array $disk_info): void {
    $site_name = $this->configFactory->get('system.site')->get('name');
    $module = 'disk_space_alert';
    $key = 'disk_space_alert';
    $to = $this->configFactory->get('system.site')->get('mail');
    $params['message'] = $this->buildEmailMessage($disk_info, $site_name);
    $langcode = $this->languageManager->getCurrentLanguage()->getId();
    $send = TRUE;

    $this->mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
  }

  /**
   * Builds the email message.
   *
   * @param array $disk_info
   *   The disk space information.
   * @param string $site_name
   *   The site name.
   *
   * @return string
   *   The email message.
   */
  private function buildEmailMessage(array $disk_info, string $site_name): string {
    return $this->t(
      'Disk space alert for @site_name:

Disk Name: @diskName
Total Space: @totalSpace (@totalBytes bytes)
Available Space: @availableSpace (@availableBytes bytes)
Available Percentage: @availablePercentage%
Occupied Space: @occupiedSpace (@occupiedBytes bytes)
Occupied Percentage: @occupiedPercentage%
Threshold: @threshold%
Above Threshold Size: @aboveThresholdSize
Above Threshold Percentage: @aboveThresholdPercentage%
Timestamp: @timestamp',
      [
        '@site_name' => $site_name,
        '@diskName' => $disk_info['disk_name'],
        '@totalSpace' => $disk_info['total_space'],
        '@totalBytes' => $disk_info['total_space_bytes'],
        '@availableSpace' => $disk_info['free_space'],
        '@availableBytes' => $disk_info['free_space_bytes'],
        '@availablePercentage' => $disk_info['available_percentage'],
        '@occupiedSpace' => $disk_info['used_space'],
        '@occupiedBytes' => $disk_info['used_space_bytes'],
        '@occupiedPercentage' => $disk_info['used_percentage'],
        '@threshold' => $disk_info['threshold'],
        '@aboveThresholdSize' => ByteSizeMarkup::create($disk_info['above_threshold_size'], $this->languageManager->getDefaultLanguage()->getId())->render(),
        '@aboveThresholdPercentage' => $disk_info['above_threshold_percentage'],
        '@timestamp' => date('c', $disk_info['timestamp']),
      ]
    );
  }

  /**
   * Sends a POST request with the disk space information.
   *
   * @param array $disk_info
   *   The disk space information.
   * @param string $url
   *   The URL to send the POST request to.
   * @param string $message
   *   The POST message.
   */
  private function sendPostRequest(array $disk_info, string $url, string $message): void {
    $tokens = [
      '@diskName' => $disk_info['disk_name'],
      '@totalSpace' => $disk_info['total_space'],
      '@totalBytes' => $disk_info['total_space_bytes'],
      '@availableSpace' => $disk_info['free_space'],
      '@availableBytes' => $disk_info['free_space_bytes'],
      '@availablePercentage' => $disk_info['available_percentage'],
      '@occupiedSpace' => $disk_info['used_space'],
      '@occupiedBytes' => $disk_info['used_space_bytes'],
      '@occupiedPercentage' => $disk_info['used_percentage'],
    // As number.
      '@threshold' => $disk_info['threshold'],
    // As boolean.
      '@isAboveThreshold' => $disk_info['is_above_threshold'] ? 'true' : 'false',
      '@aboveThresholdSize' => ByteSizeMarkup::create($disk_info['above_threshold_size'], $this->languageManager->getDefaultLanguage()->getId())->render(),
      '@aboveThresholdBytes' => $disk_info['above_threshold_size'],
      '@aboveThresholdPercentage' => $disk_info['above_threshold_percentage'],
      '@timestamp' => date('c', $disk_info['timestamp']),
    ];

    foreach ($tokens as $token => $value) {
      $message = str_replace($token, $value, $message);
    }

    try {
      $response = $this->httpClient->request(
        'POST',
        $url,
        [
          'body' => $message,
          'headers' => [
            'Content-Type' => $this->determineContentType($message),
          ],
        ]
      );

      $this->logger->info(
        'POST request sent to @url with message: @message',
        [
          '@url' => $url,
          '@message' => $message,
        ]
      );

      $this->logger->info(
        'POST request response: @response',
        [
          '@response' => $response->getBody()->getContents(),
        ]
      );
    }
    catch (\Exception $e) {
      $this->logger->error(
        'Failed to send POST request: @message',
        [
          '@message' => $e->getMessage(),
        ]
          );
    }
  }

  /**
   * Determines the content type of the message.
   *
   * @param string $message
   *   The message.
   *
   * @return string
   *   The content type.
   */
  private function determineContentType(string $message): string {
    json_decode($message);
    if (json_last_error() === JSON_ERROR_NONE) {
      return 'application/json';
    }

    libxml_use_internal_errors(TRUE);
    simplexml_load_string($message);
    if (libxml_get_errors()) {
      libxml_clear_errors();
      return 'text/plain';
    }

    return 'application/xml';
  }

}
